@@ -61,10 +61,11 @@ TArea::TArea(TMap * map , TRoomDB * pRDB )
 
 TArea::~TArea()
 {
-    if (mpRoomDB)
+    if (mpRoomDB) {
         mpRoomDB->removeArea((TArea*)this);
-    else
+    } else {
         qDebug() << "ERROR: In TArea::~TArea(), instance has no mpRoomDB";
+    }
 }
 
 int TArea::getAreaID()
@@ -549,8 +550,7 @@ const QMultiMap<int, QPair<QString, int>> TArea::getAreaExitRoomData() const
         case DIR_OUT:       exitData.first = QString("out");                           break;
         case DIR_OTHER:     roomsWithOtherAreaSpecialExits.insert(itAreaExit.key());   break;
         default:
-            qWarning("TArea::getAreaExitRoomData() Warning: unrecognised exit code %i found for exit from room %i to room %i.",
-                     itAreaExit.value().second, itAreaExit.key(), itAreaExit.value().first);
+            qWarning("TArea::getAreaExitRoomData() Warning: unrecognised exit code %i found for exit from room %i to room %i.", itAreaExit.value().second, itAreaExit.key(), itAreaExit.value().first);
         }
         if (!exitData.first.isEmpty()) {
             results.insert(itAreaExit.key(), exitData);
